@@ -549,6 +549,23 @@ public class TestRepository<R extends Repository> {
 		return new BranchBuilder(ref);
 	}
 
+	/**
+	 * Tag an object using a lightweight tag.
+	 *
+	 * @param name
+	 *            the tag name. The /refs/tags/ prefix will be added if the name
+	 *            doesn't start with it
+	 * @param obj
+	 *            the object to tag
+	 * @return the tagged object
+	 * @throws Exception
+	 */
+	public ObjectId lightweightTag(String name, ObjectId obj) throws Exception {
+		if (!name.startsWith(Constants.R_TAGS))
+			name = Constants.R_TAGS + name;
+		return update(name, obj);
+	}
+
 	/**
 	 * Run consistency checks against the object database.
 	 * <p>
@@ -733,6 +750,8 @@ public class TestRepository<R extends Repository> {
 
 		private final DirCache tree = DirCache.newInCore();
 
+		private ObjectId topLevelTree;
+
 		private final List<RevCommit> parents = new ArrayList<RevCommit>(2);
 
 		private int tick = 1;
@@ -787,20 +806,29 @@ public class TestRepository<R extends Repository> {
 			return this;
 		}
 
+		public CommitBuilder setTopLevelTree(ObjectId treeId) {
+			topLevelTree = treeId;
+			return this;
+		}
+
 		public CommitBuilder add(String path, String content) throws Exception {
 			return add(path, blob(content));
 		}
 
 		public CommitBuilder add(String path, final RevBlob id)
 				throws Exception {
-			DirCacheEditor e = tree.editor();
-			e.add(new PathEdit(path) {
+			return edit(new PathEdit(path) {
 				@Override
 				public void apply(DirCacheEntry ent) {
 					ent.setFileMode(FileMode.REGULAR_FILE);
 					ent.setObjectId(id);
 				}
 			});
+		}
+
+		public CommitBuilder edit(PathEdit edit) {
+			DirCacheEditor e = tree.editor();
+			e.add(edit);
 			e.finish();
 			return this;
 		}
@@ -836,7 +864,10 @@ public class TestRepository<R extends Repository> {
 
 				ObjectId commitId;
 				try {
-					c.setTreeId(tree.writeTree(inserter));
+					if (topLevelTree != null)
+						c.setTreeId(topLevelTree);
+					else
+						c.setTreeId(tree.writeTree(inserter));
 					commitId = inserter.insert(c);
 					inserter.flush();
 				} finally {
