@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011, Chris Aniszczyk <caniszczyk@gmail.com>
+ * Copyright (C) 2011-2012, Chris Aniszczyk <caniszczyk@gmail.com>
  * and other copyright owners as documented in the project's IP log.
  *
  * This program and the accompanying materials are made available
@@ -46,6 +46,7 @@ import static org.eclipse.jgit.api.ResetCommand.ResetType.HARD;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
@@ -140,7 +141,7 @@ public class ResetCommandTest extends RepositoryTestCase {
 				.call();
 		// check if HEAD points to initial commit now
 		ObjectId head = db.resolve(Constants.HEAD);
-		assertTrue(head.equals(initialCommit));
+		assertEquals(initialCommit, head);
 		// check if files were removed
 		assertFalse(indexFile.exists());
 		assertTrue(untrackedFile.exists());
@@ -177,7 +178,7 @@ public class ResetCommandTest extends RepositoryTestCase {
 				.call();
 		// check if HEAD points to initial commit now
 		ObjectId head = db.resolve(Constants.HEAD);
-		assertTrue(head.equals(initialCommit));
+		assertEquals(initialCommit, head);
 		// check if files still exist
 		assertTrue(untrackedFile.exists());
 		assertTrue(indexFile.exists());
@@ -198,7 +199,7 @@ public class ResetCommandTest extends RepositoryTestCase {
 				.call();
 		// check if HEAD points to initial commit now
 		ObjectId head = db.resolve(Constants.HEAD);
-		assertTrue(head.equals(initialCommit));
+		assertEquals(initialCommit, head);
 		// check if files still exist
 		assertTrue(untrackedFile.exists());
 		assertTrue(indexFile.exists());
@@ -276,7 +277,7 @@ public class ResetCommandTest extends RepositoryTestCase {
 
 		// check that HEAD hasn't moved
 		ObjectId head = db.resolve(Constants.HEAD);
-		assertTrue(head.equals(secondCommit));
+		assertEquals(secondCommit, head);
 		// check if files still exist
 		assertTrue(untrackedFile.exists());
 		assertTrue(indexFile.exists());
@@ -305,7 +306,7 @@ public class ResetCommandTest extends RepositoryTestCase {
 
 		// check that HEAD hasn't moved
 		ObjectId head = db.resolve(Constants.HEAD);
-		assertTrue(head.equals(secondCommit));
+		assertEquals(secondCommit, head);
 		// check if files still exist
 		assertTrue(untrackedFile.exists());
 		assertTrue(inHead("dir/b.txt"));
@@ -331,7 +332,7 @@ public class ResetCommandTest extends RepositoryTestCase {
 
 		// check that HEAD hasn't moved
 		ObjectId head = db.resolve(Constants.HEAD);
-		assertTrue(head.equals(secondCommit));
+		assertEquals(secondCommit, head);
 		// check if files still exist
 		assertTrue(untrackedFile.exists());
 		assertTrue(indexFile.exists());
@@ -356,7 +357,38 @@ public class ResetCommandTest extends RepositoryTestCase {
 		git.reset().setRef(tagName).setMode(HARD).call();
 
 		ObjectId head = db.resolve(Constants.HEAD);
-		assertTrue(head.equals(secondCommit));
+		assertEquals(secondCommit, head);
+	}
+
+	@Test
+	public void testHardResetAfterSquashMerge() throws Exception {
+		Git g = new Git(db);
+
+		writeTrashFile("file1", "file1");
+		g.add().addFilepattern("file1").call();
+		RevCommit first = g.commit().setMessage("initial commit").call();
+
+		assertTrue(new File(db.getWorkTree(), "file1").exists());
+		createBranch(first, "refs/heads/branch1");
+		checkoutBranch("refs/heads/branch1");
+
+		writeTrashFile("file2", "file2");
+		g.add().addFilepattern("file2").call();
+		g.commit().setMessage("second commit").call();
+		assertTrue(new File(db.getWorkTree(), "file2").exists());
+
+		checkoutBranch("refs/heads/master");
+
+		MergeResult result = g.merge().include(db.getRef("branch1"))
+				.setSquash(true).call();
+
+		assertEquals(MergeResult.MergeStatus.FAST_FORWARD_SQUASHED,
+				result.getMergeStatus());
+		assertNotNull(db.readSquashCommitMsg());
+
+		g.reset().setMode(ResetType.HARD).setRef(first.getName()).call();
+
+		assertNull(db.readSquashCommitMsg());
 	}
 
 	private void assertReflog(ObjectId prevHead, ObjectId head)
