@@ -78,7 +78,7 @@ public class SubmoduleAddTest extends RepositoryTestCase {
 	@Test
 	public void commandWithNullPath() throws GitAPIException {
 		try {
-			new SubmoduleAddCommand(db).setURI("uri").call();
+			new SubmoduleAddCommand(db).setURI("uri").call().close();
 			fail("Exception not thrown");
 		} catch (IllegalArgumentException e) {
 			assertEquals(JGitText.get().pathNotConfigured, e.getMessage());
@@ -88,7 +88,8 @@ public class SubmoduleAddTest extends RepositoryTestCase {
 	@Test
 	public void commandWithEmptyPath() throws GitAPIException {
 		try {
-			new SubmoduleAddCommand(db).setPath("").setURI("uri").call();
+			new SubmoduleAddCommand(db).setPath("").setURI("uri").call()
+					.close();
 			fail("Exception not thrown");
 		} catch (IllegalArgumentException e) {
 			assertEquals(JGitText.get().pathNotConfigured, e.getMessage());
@@ -98,7 +99,7 @@ public class SubmoduleAddTest extends RepositoryTestCase {
 	@Test
 	public void commandWithNullUri() throws GitAPIException {
 		try {
-			new SubmoduleAddCommand(db).setPath("sub").call();
+			new SubmoduleAddCommand(db).setPath("sub").call().close();
 			fail("Exception not thrown");
 		} catch (IllegalArgumentException e) {
 			assertEquals(JGitText.get().uriNotConfigured, e.getMessage());
@@ -108,7 +109,8 @@ public class SubmoduleAddTest extends RepositoryTestCase {
 	@Test
 	public void commandWithEmptyUri() throws GitAPIException {
 		try {
-			new SubmoduleAddCommand(db).setPath("sub").setURI("").call();
+			new SubmoduleAddCommand(db).setPath("sub").setURI("").call()
+					.close();
 			fail("Exception not thrown");
 		} catch (IllegalArgumentException e) {
 			assertEquals(JGitText.get().uriNotConfigured, e.getMessage());
@@ -129,6 +131,7 @@ public class SubmoduleAddTest extends RepositoryTestCase {
 		command.setURI(uri);
 		Repository repo = command.call();
 		assertNotNull(repo);
+		addRepoToClose(repo);
 
 		SubmoduleWalk generator = SubmoduleWalk.forIndex(db);
 		assertTrue(generator.next());
@@ -137,7 +140,9 @@ public class SubmoduleAddTest extends RepositoryTestCase {
 		assertEquals(uri, generator.getModulesUrl());
 		assertEquals(path, generator.getModulesPath());
 		assertEquals(uri, generator.getConfigUrl());
-		assertNotNull(generator.getRepository());
+		Repository subModRepo = generator.getRepository();
+		addRepoToClose(subModRepo);
+		assertNotNull(subModRepo);
 		assertEquals(commit, repo.resolve(Constants.HEAD));
 
 		Status status = Git.wrap(db).status().call();
@@ -165,7 +170,7 @@ public class SubmoduleAddTest extends RepositoryTestCase {
 		command.setPath(path);
 		command.setURI("git://server/repo.git");
 		try {
-			command.call();
+			command.call().close();
 			fail("Exception not thrown");
 		} catch (JGitInternalException e) {
 			assertEquals(
@@ -188,6 +193,7 @@ public class SubmoduleAddTest extends RepositoryTestCase {
 		command.setURI(uri);
 		Repository repo = command.call();
 		assertNotNull(repo);
+		addRepoToClose(repo);
 
 		SubmoduleWalk generator = SubmoduleWalk.forIndex(db);
 		assertTrue(generator.next());
@@ -199,11 +205,12 @@ public class SubmoduleAddTest extends RepositoryTestCase {
 		if (File.separatorChar == '\\')
 			fullUri = fullUri.replace('\\', '/');
 		assertEquals(fullUri, generator.getConfigUrl());
-		assertNotNull(generator.getRepository());
+		Repository subModRepo = generator.getRepository();
+		addRepoToClose(subModRepo);
+		assertNotNull(subModRepo);
 		assertEquals(
 				fullUri,
-				generator
-						.getRepository()
+				subModRepo
 						.getConfig()
 						.getString(ConfigConstants.CONFIG_REMOTE_SECTION,
 								Constants.DEFAULT_REMOTE_NAME,
@@ -238,7 +245,9 @@ public class SubmoduleAddTest extends RepositoryTestCase {
 		command.setPath(path2);
 		String url2 = db.getDirectory().toURI().toString();
 		command.setURI(url2);
-		assertNotNull(command.call());
+		Repository r = command.call();
+		assertNotNull(r);
+		addRepoToClose(r);
 
 		modulesConfig.load();
 		assertEquals(path1, modulesConfig.getString(
