@@ -115,6 +115,7 @@ public class SubmoduleSyncTest extends RepositoryTestCase {
 				.setURI(db.getDirectory().toURI().toString())
 				.setDirectory(new File(db.getWorkTree(), path)).call()
 				.getRepository();
+		addRepoToClose(subRepo);
 		assertNotNull(subRepo);
 
 		SubmoduleWalk generator = SubmoduleWalk.forIndex(db);
@@ -133,7 +134,9 @@ public class SubmoduleSyncTest extends RepositoryTestCase {
 		generator = SubmoduleWalk.forIndex(db);
 		assertTrue(generator.next());
 		assertEquals(url, generator.getConfigUrl());
-		StoredConfig submoduleConfig = generator.getRepository().getConfig();
+		Repository subModRepository = generator.getRepository();
+		addRepoToClose(subModRepository);
+		StoredConfig submoduleConfig = subModRepository.getConfig();
 		assertEquals(url, submoduleConfig.getString(
 				ConfigConstants.CONFIG_REMOTE_SECTION,
 				Constants.DEFAULT_REMOTE_NAME, ConfigConstants.CONFIG_KEY_URL));
@@ -181,6 +184,7 @@ public class SubmoduleSyncTest extends RepositoryTestCase {
 				.setDirectory(new File(db.getWorkTree(), path)).call()
 				.getRepository();
 		assertNotNull(subRepo);
+		addRepoToClose(subRepo);
 
 		SubmoduleWalk generator = SubmoduleWalk.forIndex(db);
 		assertTrue(generator.next());
@@ -202,7 +206,9 @@ public class SubmoduleSyncTest extends RepositoryTestCase {
 		generator = SubmoduleWalk.forIndex(db);
 		assertTrue(generator.next());
 		assertEquals("git://server/sub.git", generator.getConfigUrl());
-		StoredConfig submoduleConfig = generator.getRepository().getConfig();
+		Repository subModRepository1 = generator.getRepository();
+		addRepoToClose(subModRepository1);
+		StoredConfig submoduleConfig = subModRepository1.getConfig();
 		assertEquals("git://server/sub.git", submoduleConfig.getString(
 				ConfigConstants.CONFIG_REMOTE_SECTION,
 				Constants.DEFAULT_REMOTE_NAME, ConfigConstants.CONFIG_KEY_URL));
