@@ -46,6 +46,7 @@ package org.eclipse.jgit.api;
 import java.io.File;
 import java.io.IOException;
 import java.text.MessageFormat;
+import java.util.ArrayList;
 import java.util.LinkedList;
 import java.util.List;
 
@@ -78,6 +79,7 @@ import org.eclipse.jgit.revwalk.RevTree;
 import org.eclipse.jgit.revwalk.RevWalk;
 import org.eclipse.jgit.treewalk.TreeWalk;
 import org.eclipse.jgit.treewalk.filter.PathFilterGroup;
+import org.eclipse.jgit.util.FileUtils;
 
 /**
  * Checkout a branch to the working tree
@@ -134,7 +136,7 @@ public class CheckoutCommand extends GitCommand<Ref> {
 		try {
 			if (checkoutAllPaths || !paths.isEmpty()) {
 				checkoutPaths();
-				status = CheckoutResult.OK_RESULT;
+				status = new CheckoutResult(Status.OK, paths);
 				setCallable(false);
 				return null;
 			}
@@ -214,12 +216,14 @@ public class CheckoutCommand extends GitCommand<Ref> {
 				throw new JGitInternalException(MessageFormat.format(JGitText
 						.get().checkoutUnexpectedResult, updateResult.name()));
 
+
 			if (!dco.getToBeDeleted().isEmpty()) {
-				status = new CheckoutResult(Status.NONDELETED, dco
-						.getToBeDeleted());
-			}
-			else
-				status = CheckoutResult.OK_RESULT;
+				status = new CheckoutResult(Status.NONDELETED,
+						dco.getToBeDeleted());
+			} else
+				status = new CheckoutResult(new ArrayList<String>(dco
+						.getUpdated().keySet()), dco.getRemoved());
+
 			return ref;
 		} catch (IOException ioe) {
 			throw new JGitInternalException(ioe.getMessage(), ioe);
@@ -297,9 +301,11 @@ public class CheckoutCommand extends GitCommand<Ref> {
 						public void apply(DirCacheEntry ent) {
 							ent.setObjectId(blobId);
 							ent.setFileMode(mode);
+							File file = new File(workTree, ent.getPathString());
+							File parentDir = file.getParentFile();
 							try {
-								DirCacheCheckout.checkoutEntry(repo, new File(
-										workTree, ent.getPathString()), ent, r);
+								FileUtils.mkdirs(parentDir, true);
+								DirCacheCheckout.checkoutEntry(repo, file, ent, r);
 							} catch (IOException e) {
 								throw new JGitInternalException(
 										MessageFormat.format(
