@@ -137,6 +137,8 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 
 	private static final String ONTO = "onto";
 
+	private static final String ONTO_NAME = "onto-name";
+
 	private static final String PATCH = "patch";
 
 	private static final String REBASE_HEAD = "head";
@@ -167,6 +169,8 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 
 	private RevCommit upstreamCommit;
 
+	private String upstreamCommitName;
+
 	private ProgressMonitor monitor = NullProgressMonitor.INSTANCE;
 
 	private final RevWalk walk;
@@ -211,9 +215,16 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 			case SKIP:
 				// fall through
 			case CONTINUE:
-				String upstreamCommitName = readFile(rebaseDir, ONTO);
+				String upstreamCommitId = readFile(rebaseDir, ONTO);
+				try {
+					upstreamCommitName = readFile(rebaseDir, ONTO_NAME);
+				} catch (FileNotFoundException e) {
+					// Fall back to commit ID if file doesn't exist (e.g. rebase
+					// was started by C Git)
+					upstreamCommitName = upstreamCommitId;
+				}
 				this.upstreamCommit = walk.parseCommit(repo
-						.resolve(upstreamCommitName));
+						.resolve(upstreamCommitId));
 				break;
 			case BEGIN:
 				RebaseResult res = initFilesAndRewind();
@@ -267,8 +278,10 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 						// TODO if the content of this commit is already merged
 						// here we should skip this step in order to avoid
 						// confusing pseudo-changed
+						String ourCommitName = getOurCommitName();
 						CherryPickResult cherryPickResult = new Git(repo)
-								.cherryPick().include(commitToPick).call();
+								.cherryPick().include(commitToPick)
+								.setOurCommitName(ourCommitName).call();
 						switch (cherryPickResult.getStatus()) {
 						case FAILED:
 							if (operation == Operation.BEGIN)
@@ -300,6 +313,14 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		}
 	}
 
+	private String getOurCommitName() {
+		// If onto is different from upstream, this should say "onto", but
+		// RebaseCommand doesn't support a different "onto" at the moment.
+		String ourCommitName = "Upstream, based on "
+				+ Repository.shortenRefName(upstreamCommitName);
+		return ourCommitName;
+	}
+
 	private void updateHead(String headName, RevCommit newHead)
 			throws IOException {
 		// point the previous head (if any) to the new commit
@@ -439,6 +460,7 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		// representation for date and timezone
 		sb.append(GIT_AUTHOR_DATE);
 		sb.append("='");
+		sb.append("@"); // @ for time in seconds since 1970
 		String externalString = author.toExternalString();
 		sb
 				.append(externalString.substring(externalString
@@ -583,6 +605,7 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		createFile(rebaseDir, REBASE_HEAD, headId.name());
 		createFile(rebaseDir, HEAD_NAME, headName);
 		createFile(rebaseDir, ONTO, upstreamCommit.name());
+		createFile(rebaseDir, ONTO_NAME, upstreamCommitName);
 		createFile(rebaseDir, INTERACTIVE, "");
 		BufferedWriter fw = new BufferedWriter(new OutputStreamWriter(
 				new FileOutputStream(new File(rebaseDir, GIT_REBASE_TODO)),
@@ -700,6 +723,9 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 			// these operations are only possible while in a rebasing state
 			switch (repo.getRepositoryState()) {
 			case REBASING_INTERACTIVE:
+			case REBASING:
+			case REBASING_REBASING:
+			case REBASING_MERGE:
 				break;
 			default:
 				throw new WrongRepositoryStateException(MessageFormat.format(
@@ -824,7 +850,7 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		return true;
 	}
 
-	private List<Step> loadSteps() throws IOException {
+	List<Step> loadSteps() throws IOException {
 		byte[] buf = IO.readFully(new File(rebaseDir, GIT_REBASE_TODO));
 		int ptr = 0;
 		int tokenBegin = 0;
@@ -832,13 +858,12 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		while (ptr < buf.length) {
 			tokenBegin = ptr;
 			ptr = RawParseUtils.nextLF(buf, ptr);
-			int nextSpace = 0;
+			int nextSpace = RawParseUtils.next(buf, tokenBegin, ' ');
 			int tokenCount = 0;
 			Step current = null;
 			while (tokenCount < 3 && nextSpace < ptr) {
 				switch (tokenCount) {
 				case 0:
-					nextSpace = RawParseUtils.next(buf, tokenBegin, ' ');
 					String actionToken = new String(buf, tokenBegin, nextSpace
 							- tokenBegin - 1);
 					tokenBegin = nextSpace;
@@ -884,6 +909,7 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 	 */
 	public RebaseCommand setUpstream(RevCommit upstream) {
 		this.upstreamCommit = upstream;
+		this.upstreamCommitName = upstream.name();
 		return this;
 	}
 
@@ -895,6 +921,7 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 	public RebaseCommand setUpstream(AnyObjectId upstream) {
 		try {
 			this.upstreamCommit = walk.parseCommit(upstream);
+			this.upstreamCommitName = upstream.name();
 		} catch (IOException e) {
 			throw new JGitInternalException(MessageFormat.format(
 					JGitText.get().couldNotReadObjectWhileParsingCommit,
@@ -917,12 +944,30 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 				throw new RefNotFoundException(MessageFormat.format(JGitText
 						.get().refNotResolved, upstream));
 			upstreamCommit = walk.parseCommit(repo.resolve(upstream));
+			upstreamCommitName = upstream;
 			return this;
 		} catch (IOException ioe) {
 			throw new JGitInternalException(ioe.getMessage(), ioe);
 		}
 	}
 
+	/**
+	 * Optionally override the name of the upstream. If this is used, it has to
+	 * come after any {@link #setUpstream} call.
+	 *
+	 * @param upstreamName
+	 *            the name which will be used to refer to upstream in conflicts
+	 * @return {@code this}
+	 */
+	public RebaseCommand setUpstreamName(String upstreamName) {
+		if (upstreamCommit == null) {
+			throw new IllegalStateException(
+					"setUpstreamName must be called after setUpstream.");
+		}
+		this.upstreamCommitName = upstreamName;
+		return this;
+	}
+
 	/**
 	 * @param operation
 	 *            the operation to perform
@@ -956,6 +1001,11 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 			return this.token;
 		}
 
+		@Override
+		public String toString() {
+			return "Action[" + token + "]";
+		}
+
 		static Action parse(String token) {
 			if (token.equals("pick") || token.equals("p"))
 				return PICK;
@@ -975,6 +1025,15 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		Step(Action action) {
 			this.action = action;
 		}
+
+		@Override
+		public String toString() {
+			return "Step[" + action + ", "
+					+ ((commit == null) ? "null" : commit)
+					+ ", "
+					+ ((shortMessage == null) ? "null" : new String(
+							shortMessage)) + "]";
+		}
 	}
 
 	PersonIdent parseAuthor(byte[] raw) {
@@ -1000,7 +1059,13 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		String time = keyValueMap.get(GIT_AUTHOR_DATE);
 
 		// the time is saved as <seconds since 1970> <timezone offset>
-		long when = Long.parseLong(time.substring(0, time.indexOf(' '))) * 1000;
+		int timeStart = 0;
+		if (time.startsWith("@"))
+			timeStart = 1;
+		else
+			timeStart = 0;
+		long when = Long
+				.parseLong(time.substring(timeStart, time.indexOf(' '))) * 1000;
 		String tzOffsetString = time.substring(time.indexOf(' ') + 1);
 		int multiplier = -1;
 		if (tzOffsetString.charAt(0) == '+')
