@@ -164,29 +164,35 @@ public class SubmoduleUpdateCommand extends
 					submoduleRepo = clone.call().getRepository();
 				}
 
-				RevWalk walk = new RevWalk(submoduleRepo);
-				RevCommit commit = walk.parseCommit(generator.getObjectId());
+				try {
+					RevWalk walk = new RevWalk(submoduleRepo);
+					RevCommit commit = walk
+							.parseCommit(generator.getObjectId());
 
-				String update = generator.getConfigUpdate();
-				if (ConfigConstants.CONFIG_KEY_MERGE.equals(update)) {
-					MergeCommand merge = new MergeCommand(submoduleRepo);
-					merge.include(commit);
-					merge.call();
-				} else if (ConfigConstants.CONFIG_KEY_REBASE.equals(update)) {
-					RebaseCommand rebase = new RebaseCommand(submoduleRepo);
-					rebase.setUpstream(commit);
-					rebase.call();
-				} else {
-					// Checkout commit referenced in parent repository's index
-					// as a detached HEAD
-					DirCacheCheckout co = new DirCacheCheckout(submoduleRepo,
-							submoduleRepo.lockDirCache(), commit.getTree());
-					co.setFailOnConflict(true);
-					co.checkout();
-					RefUpdate refUpdate = submoduleRepo.updateRef(
-							Constants.HEAD, true);
-					refUpdate.setNewObjectId(commit);
-					refUpdate.forceUpdate();
+					String update = generator.getConfigUpdate();
+					if (ConfigConstants.CONFIG_KEY_MERGE.equals(update)) {
+						MergeCommand merge = new MergeCommand(submoduleRepo);
+						merge.include(commit);
+						merge.call();
+					} else if (ConfigConstants.CONFIG_KEY_REBASE.equals(update)) {
+						RebaseCommand rebase = new RebaseCommand(submoduleRepo);
+						rebase.setUpstream(commit);
+						rebase.call();
+					} else {
+						// Checkout commit referenced in parent repository's
+						// index as a detached HEAD
+						DirCacheCheckout co = new DirCacheCheckout(
+								submoduleRepo, submoduleRepo.lockDirCache(),
+								commit.getTree());
+						co.setFailOnConflict(true);
+						co.checkout();
+						RefUpdate refUpdate = submoduleRepo.updateRef(
+								Constants.HEAD, true);
+						refUpdate.setNewObjectId(commit);
+						refUpdate.forceUpdate();
+					}
+				} finally {
+					submoduleRepo.close();
 				}
 				updated.add(generator.getPath());
 			}
