@@ -332,8 +332,14 @@ public class DirCacheCheckout {
 					// conflict
 					update(m.getEntryPathString(), m.getEntryObjectId(),
 							m.getEntryFileMode());
-				else
-					keep(i.getDirCacheEntry());
+				else {
+					// update the timestamp of the index with the one from the
+					// file if not set, as we are sure to be in sync here.
+					DirCacheEntry entry = i.getDirCacheEntry();
+					if (entry.getLastModified() == 0)
+						entry.setLastModified(f.getEntryLastModified());
+					keep(entry);
+				}
 			} else
 				// The index contains a folder
 				keep(i.getDirCacheEntry());
@@ -546,8 +552,8 @@ public class DirCacheCheckout {
 		 *      ------------------------------------------------------------------
 		 * 1    D        D       F       Y         N       Y       N           Update
 		 * 2    D        D       F       N         N       Y       N           Conflict
-		 * 3    D        F       D                 Y       N       N           Update
-		 * 4    D        F       D                 N       N       N           Update
+		 * 3    D        F       D                 Y       N       N           Keep
+		 * 4    D        F       D                 N       N       N           Conflict
 		 * 5    D        F       F       Y         N       N       Y           Keep
 		 * 6    D        F       F       N         N       N       Y           Keep
 		 * 7    F        D       F       Y         Y       N       N           Update
@@ -609,10 +615,7 @@ public class DirCacheCheckout {
 
 				break;
 			case 0xDFD: // 3 4
-				// CAUTION: I put it into removed instead of updated, because
-				// that's what our tests expect
-				// updated.put(name, mId);
-				remove(name);
+				keep(dce);
 				break;
 			case 0xF0D: // 18
 				remove(name);
@@ -697,12 +700,13 @@ public class DirCacheCheckout {
 
 			/**
 			 * <pre>
-			 * 		    I (index)                H        M        Result
-			 * 	        -------------------------------------------------------
-			 * 	        0 nothing             nothing  nothing  (does not happen)
-			 * 	        1 nothing             nothing  exists   use M
-			 * 	        2 nothing             exists   nothing  remove path from index
-			 * 	        3 nothing             exists   exists   use M
+			 * 	          I (index)     H        M     H==M  Result
+			 * 	        -------------------------------------------
+			 * 	        0 nothing    nothing  nothing        (does not happen)
+			 * 	        1 nothing    nothing  exists         use M
+			 * 	        2 nothing    exists   nothing        remove path from index
+			 * 	        3 nothing    exists   exists   yes   keep index
+			 * 	          nothing    exists   exists   no    fail
 			 * </pre>
 			 */
 
@@ -710,8 +714,12 @@ public class DirCacheCheckout {
 				update(name, mId, mMode); // 1
 			else if (m == null)
 				remove(name); // 2
-			else
-				update(name, mId, mMode); // 3
+			else { // 3
+				if (equalIdAndMode(hId, hMode, mId, mMode))
+					keep(dce);
+				else
+					conflict(name, dce, h, m);
+			}
 		} else {
 			if (h == null) {
 				/**
@@ -956,6 +964,7 @@ public class DirCacheCheckout {
 			DirCacheEntry entry, ObjectReader or) throws IOException {
 		ObjectLoader ol = or.open(entry.getObjectId());
 		File parentDir = f.getParentFile();
+		parentDir.mkdirs();
 		File tmpFile = File.createTempFile("._" + f.getName(), null, parentDir);
 		WorkingTreeOptions opt = repo.getConfig().get(WorkingTreeOptions.KEY);
 		FileOutputStream rawChannel = new FileOutputStream(tmpFile);
@@ -1015,9 +1024,8 @@ public class DirCacheCheckout {
 	}
 
 	private static boolean isValidPathSegment(CanonicalTreeParser t) {
-		String osName = SystemReader.getInstance().getProperty("os.name");
-		boolean isWindows = "Windows".equals(osName);
-		boolean isOSX = "Darwin".equals(osName) || "Mac OS X".equals(osName);
+		boolean isWindows = SystemReader.getInstance().isWindows();
+		boolean isOSX = SystemReader.getInstance().isMacOS();
 		boolean ignCase = isOSX || isWindows;
 
 		int ptr = t.getNameOffset();
