@@ -1660,7 +1660,8 @@ public class PackWriter {
 
 				for (int i = 0; i < cmit.getParentCount(); i++) {
 					RevCommit p = cmit.getParent(i);
-					if (!p.has(added) && !p.has(RevFlag.UNINTERESTING)) {
+					if (!p.has(added) && !p.has(RevFlag.UNINTERESTING)
+							&& !exclude(p)) {
 						p.add(added);
 						addObject(p, 0);
 						commitCnt++;
@@ -1869,6 +1870,7 @@ public class PackWriter {
 			otp.clearReuseAsIs();
 		}
 
+		otp.setDeltaAttempted(next.wasDeltaAttempted());
 		otp.select(next);
 	}
 
