@@ -67,6 +67,7 @@ import java.util.concurrent.TimeUnit;
 import org.eclipse.jgit.errors.MissingObjectException;
 import org.eclipse.jgit.errors.PackProtocolException;
 import org.eclipse.jgit.internal.JGitText;
+import org.eclipse.jgit.lib.BatchRefUpdate;
 import org.eclipse.jgit.lib.Config;
 import org.eclipse.jgit.lib.Config.SectionParser;
 import org.eclipse.jgit.lib.Constants;
@@ -135,10 +136,10 @@ public abstract class BaseReceivePack {
 	}
 
 	/** Database we write the stored objects into. */
-	protected final Repository db;
+	private final Repository db;
 
 	/** Revision traversal support over {@link #db}. */
-	protected final RevWalk walk;
+	private final RevWalk walk;
 
 	/**
 	 * Is the client connection a bi-directional socket or pipe?
@@ -151,19 +152,22 @@ public abstract class BaseReceivePack {
 	 * If false, this class runs in a read everything then output results mode,
 	 * making it suitable for single round-trip systems RPCs such as HTTP.
 	 */
-	protected boolean biDirectionalPipe = true;
+	private boolean biDirectionalPipe = true;
+
+	/** Expecting data after the pack footer */
+	private boolean expectDataAfterPackFooter;
 
 	/** Should an incoming transfer validate objects? */
-	protected boolean checkReceivedObjects;
+	private boolean checkReceivedObjects;
 
 	/** Should an incoming transfer permit create requests? */
-	protected boolean allowCreates;
+	private boolean allowCreates;
 
 	/** Should an incoming transfer permit delete requests? */
-	protected boolean allowDeletes;
+	private boolean allowDeletes;
 
 	/** Should an incoming transfer permit non-fast-forward requests? */
-	protected boolean allowNonFastForwards;
+	private boolean allowNonFastForwards;
 
 	private boolean allowOfsDelta;
 
@@ -208,23 +212,20 @@ public abstract class BaseReceivePack {
 	private PackParser parser;
 
 	/** The refs we advertised as existing at the start of the connection. */
-	protected Map<String, Ref> refs;
+	private Map<String, Ref> refs;
 
 	/** All SHA-1s shown to the client, which can be possible edges. */
-	protected Set<ObjectId> advertisedHaves;
+	private Set<ObjectId> advertisedHaves;
 
 	/** Capabilities requested by the client. */
-	protected Set<String> enabledCapabilities;
+	private Set<String> enabledCapabilities;
 
 	private List<ReceiveCommand> commands;
 
 	private StringBuilder advertiseError;
 
-	/** If {@link BasePackPushConnection#CAPABILITY_REPORT_STATUS} is enabled. */
-	protected boolean reportStatus;
-
 	/** If {@link BasePackPushConnection#CAPABILITY_SIDE_BAND_64K} is enabled. */
-	protected boolean sideBand;
+	private boolean sideBand;
 
 	/** Lock around the received pack file, while updating refs. */
 	private PackLock packLock;
@@ -453,6 +454,19 @@ public abstract class BaseReceivePack {
 		biDirectionalPipe = twoWay;
 	}
 
+	/** @return true if there is data expected after the pack footer. */
+	public boolean isExpectDataAfterPackFooter() {
+		return expectDataAfterPackFooter;
+	}
+
+	/**
+	 * @param e
+	 *            true if there is additional data in InputStream after pack.
+	 */
+	public void setExpectDataAfterPackFooter(boolean e) {
+		expectDataAfterPackFooter = e;
+	}
+
 	/**
 	 * @return true if this instance will verify received objects are formatted
 	 *         correctly. Validating objects requires more CPU time on this side
@@ -855,9 +869,7 @@ public abstract class BaseReceivePack {
 
 	/** Enable capabilities based on a previously read capabilities line. */
 	protected void enableCapabilities() {
-		reportStatus = enabledCapabilities.contains(CAPABILITY_REPORT_STATUS);
-
-		sideBand = enabledCapabilities.contains(CAPABILITY_SIDE_BAND_64K);
+		sideBand = isCapabilityEnabled(CAPABILITY_SIDE_BAND_64K);
 		if (sideBand) {
 			OutputStream out = rawOut;
 
@@ -869,6 +881,17 @@ public abstract class BaseReceivePack {
 		}
 	}
 
+	/**
+	 * Check if the peer requested a capability.
+	 *
+	 * @param name
+	 *            protocol name identifying the capability.
+	 * @return true if the peer requested the capability to be enabled.
+	 */
+	protected boolean isCapabilityEnabled(String name) {
+		return enabledCapabilities.contains(name);
+	}
+
 	/** @return true if a pack is expected based on the list of commands. */
 	protected boolean needPack() {
 		for (final ReceiveCommand cmd : commands) {
@@ -907,7 +930,9 @@ public abstract class BaseReceivePack {
 			parser.setAllowThin(true);
 			parser.setNeedNewObjectIds(checkReferencedIsReachable);
 			parser.setNeedBaseObjectIds(checkReferencedIsReachable);
-			parser.setCheckEofAfterPackFooter(!biDirectionalPipe);
+			parser.setCheckEofAfterPackFooter(!biDirectionalPipe
+					&& !isExpectDataAfterPackFooter());
+			parser.setExpectDataAfterPackFooter(isExpectDataAfterPackFooter());
 			parser.setObjectChecking(isCheckReceivedObjects());
 			parser.setLockMessage(lockMsg);
 			parser.setMaxObjectSizeLimit(maxObjectSizeLimit);
@@ -938,7 +963,7 @@ public abstract class BaseReceivePack {
 
 		final ObjectWalk ow = new ObjectWalk(db);
 		ow.setRetainBody(false);
-		if (checkReferencedIsReachable) {
+		if (baseObjects != null) {
 			ow.sort(RevSort.TOPO);
 			if (!baseObjects.isEmpty())
 				ow.sort(RevSort.BOUNDARY, true);
@@ -955,7 +980,7 @@ public abstract class BaseReceivePack {
 			RevObject o = ow.parseAny(have);
 			ow.markUninteresting(o);
 
-			if (checkReferencedIsReachable && !baseObjects.isEmpty()) {
+			if (baseObjects != null && !baseObjects.isEmpty()) {
 				o = ow.peel(o);
 				if (o instanceof RevCommit)
 					o = ((RevCommit) o).getTree();
@@ -966,7 +991,7 @@ public abstract class BaseReceivePack {
 
 		RevCommit c;
 		while ((c = ow.next()) != null) {
-			if (checkReferencedIsReachable //
+			if (providedObjects != null //
 					&& !c.has(RevFlag.UNINTERESTING) //
 					&& !providedObjects.contains(c))
 				throw new MissingObjectException(c, Constants.TYPE_COMMIT);
@@ -977,7 +1002,7 @@ public abstract class BaseReceivePack {
 			if (o.has(RevFlag.UNINTERESTING))
 				continue;
 
-			if (checkReferencedIsReachable) {
+			if (providedObjects != null) {
 				if (providedObjects.contains(o))
 					continue;
 				else
@@ -988,7 +1013,7 @@ public abstract class BaseReceivePack {
 				throw new MissingObjectException(o, Constants.TYPE_BLOB);
 		}
 
-		if (checkReferencedIsReachable) {
+		if (baseObjects != null) {
 			for (ObjectId id : baseObjects) {
 				o = ow.parseAny(id);
 				if (!o.has(RevFlag.UNINTERESTING))
@@ -1086,11 +1111,11 @@ public abstract class BaseReceivePack {
 
 				if (oldObj instanceof RevCommit && newObj instanceof RevCommit) {
 					try {
-						if (!walk.isMergedInto((RevCommit) oldObj,
-								(RevCommit) newObj)) {
-							cmd
-									.setType(ReceiveCommand.Type.UPDATE_NONFASTFORWARD);
-						}
+						if (walk.isMergedInto((RevCommit) oldObj,
+								(RevCommit) newObj))
+							cmd.setTypeFastForwardUpdate();
+						else
+							cmd.setType(ReceiveCommand.Type.UPDATE_NONFASTFORWARD);
 					} catch (MissingObjectException e) {
 						cmd.setResult(Result.REJECTED_MISSING_OBJECT, e
 								.getMessage());
@@ -1100,6 +1125,12 @@ public abstract class BaseReceivePack {
 				} else {
 					cmd.setType(ReceiveCommand.Type.UPDATE_NONFASTFORWARD);
 				}
+
+				if (cmd.getType() == ReceiveCommand.Type.UPDATE_NONFASTFORWARD
+						&& !isAllowNonFastForwards()) {
+					cmd.setResult(Result.REJECTED_NONFASTFORWARD);
+					continue;
+				}
 			}
 
 			if (!cmd.getRefName().startsWith(Constants.R_REFS)
@@ -1123,20 +1154,30 @@ public abstract class BaseReceivePack {
 
 	/** Execute commands to update references. */
 	protected void executeCommands() {
-		List<ReceiveCommand> toApply = ReceiveCommand.filter(commands,
-				Result.NOT_ATTEMPTED);
+		List<ReceiveCommand> toApply = filterCommands(Result.NOT_ATTEMPTED);
+		if (toApply.isEmpty())
+			return;
+
 		ProgressMonitor updating = NullProgressMonitor.INSTANCE;
 		if (sideBand) {
 			SideBandProgressMonitor pm = new SideBandProgressMonitor(msgOut);
 			pm.setDelayStart(250, TimeUnit.MILLISECONDS);
 			updating = pm;
 		}
-		updating.beginTask(JGitText.get().updatingReferences, toApply.size());
-		for (ReceiveCommand cmd : toApply) {
-			updating.update(1);
-			cmd.execute(this);
+
+		BatchRefUpdate batch = db.getRefDatabase().newBatchUpdate();
+		batch.setAllowNonFastForwards(isAllowNonFastForwards());
+		batch.setRefLogIdent(getRefLogIdent());
+		batch.setRefLogMessage("push", true);
+		batch.addCommand(toApply);
+		try {
+			batch.execute(walk, updating);
+		} catch (IOException err) {
+			for (ReceiveCommand cmd : toApply) {
+				if (cmd.getResult() == Result.NOT_ATTEMPTED)
+					cmd.reject(err);
+			}
 		}
-		updating.endTask();
 	}
 
 	/**
@@ -1175,9 +1216,10 @@ public abstract class BaseReceivePack {
 			}
 
 			final StringBuilder r = new StringBuilder();
-			r.append("ng ");
-			r.append(cmd.getRefName());
-			r.append(" ");
+			if (forClient)
+				r.append("ng ").append(cmd.getRefName()).append(" ");
+			else
+				r.append(" ! [rejected] ").append(cmd.getRefName()).append(" (");
 
 			switch (cmd.getResult()) {
 			case NOT_ATTEMPTED:
@@ -1224,6 +1266,8 @@ public abstract class BaseReceivePack {
 				// We shouldn't have reached this case (see 'ok' case above).
 				continue;
 			}
+			if (!forClient)
+				r.append(")");
 			out.sendString(r.toString());
 		}
 	}
