@@ -54,8 +54,10 @@ import java.io.InputStream;
 import java.nio.ByteBuffer;
 import java.nio.channels.ReadableByteChannel;
 import java.text.MessageFormat;
+import java.util.ArrayList;
+import java.util.List;
 
-import org.eclipse.jgit.JGitText;
+import org.eclipse.jgit.internal.JGitText;
 
 /**
  * Input/Output utilities
@@ -137,17 +139,37 @@ public class IO {
 			throws FileNotFoundException, IOException {
 		final FileInputStream in = new FileInputStream(path);
 		try {
-			final long sz = in.getChannel().size();
+			long sz = Math.max(path.length(), 1);
 			if (sz > max)
 				throw new IOException(MessageFormat.format(
 						JGitText.get().fileIsTooLarge, path));
-			final byte[] buf = new byte[(int) sz];
-			int actSz = IO.readFully(in, buf, 0);
 
-			if (actSz == sz) {
-				byte[] ret = new byte[actSz];
-				System.arraycopy(buf, 0, ret, 0, actSz);
-				return ret;
+			byte[] buf = new byte[(int) sz];
+			int valid = 0;
+			for (;;) {
+				if (buf.length == valid) {
+					if (buf.length == max) {
+						int next = in.read();
+						if (next < 0)
+							break;
+
+						throw new IOException(MessageFormat.format(
+								JGitText.get().fileIsTooLarge, path));
+					}
+
+					byte[] nb = new byte[Math.min(buf.length * 2, max)];
+					System.arraycopy(buf, 0, nb, 0, valid);
+					buf = nb;
+				}
+				int n = in.read(buf, valid, buf.length - valid);
+				if (n < 0)
+					break;
+				valid += n;
+			}
+			if (valid < buf.length) {
+				byte[] nb = new byte[valid];
+				System.arraycopy(buf, 0, nb, 0, valid);
+				buf = nb;
 			}
 			return buf;
 		} finally {
@@ -311,6 +333,43 @@ public class IO {
 		}
 	}
 
+	/**
+	 * Divides the given string into lines.
+	 *
+	 * @param s
+	 *            the string to read
+	 * @return the string divided into lines
+	 * @since 2.0
+	 */
+	public static List<String> readLines(final String s) {
+		List<String> l = new ArrayList<String>();
+		StringBuilder sb = new StringBuilder();
+		for (int i = 0; i < s.length(); i++) {
+			char c = s.charAt(i);
+			if (c == '\n') {
+				l.add(sb.toString());
+				sb.setLength(0);
+				continue;
+			}
+			if (c == '\r') {
+				if (i + 1 < s.length()) {
+					c = s.charAt(++i);
+					l.add(sb.toString());
+					sb.setLength(0);
+					if (c != '\n')
+						sb.append(c);
+					continue;
+				} else { // EOF
+					l.add(sb.toString());
+					break;
+				}
+			}
+			sb.append(c);
+		}
+		l.add(sb.toString());
+		return l;
+	}
+
 	private IO() {
 		// Don't create instances of a static only utility.
 	}
